"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const Moment = __importStar(require("moment"));
const moment_range_1 = require("moment-range");
const utils_general_1 = require("@energyweb/utils-general");
const __1 = require("..");
class Entity extends utils_general_1.BlockchainDataModelEntity.Entity {
    constructor(id, configuration, isPrivate = false) {
        super(id, configuration);
        this.isPrivate = isPrivate;
        this.initialized = false;
        this.issuers = this.configuration.blockchainProperties.issuerLogicInstance;
    }
    async sync() {
        const issuer = this.isPrivate ? this.issuers.private : this.issuers.public;
        const issueRequest = await issuer.getRequestIssue(Number(this.id));
        const decodedData = await issuer.decodeIssue(issueRequest.data);
        this.owner = issueRequest.owner;
        this.fromTime = Number(decodedData['0']);
        this.toTime = Number(decodedData['1']);
        this.deviceId = decodedData['2'];
        this.approved = issueRequest.approved;
        this.revoked = issueRequest.revoked;
        this.initialized = true;
        if (this.configuration.logger) {
            this.configuration.logger.verbose(`RequestIssue ${this.id} synced`);
        }
        return this;
    }
    async approve(toAddress, value) {
        const validityData = this.configuration.blockchainProperties.web3.eth.abi.encodeFunctionCall({
            name: 'isRequestValid',
            type: 'function',
            inputs: [{
                    type: 'uint256',
                    name: '_requestId'
                }]
        }, [this.id.toString()]);
        let approveTx;
        if (this.isPrivate) {
            const commitment = {
                volume: value
            };
            const { rootHash } = this.prepareEntityCreation(commitment, __1.CommitmentSchema);
            approveTx = await this.issuers.private.approveIssue(toAddress, Number(this.id), rootHash, validityData);
        }
        else {
            approveTx = await this.issuers.public.approveIssue(toAddress, Number(this.id), Number(value), validityData);
        }
        return this.configuration.blockchainProperties.web3.utils
            .hexToNumber(approveTx.logs[1].topics[1]);
    }
    async revoke() {
        if (this.isPrivate) {
            await this.issuers.private.revokeRequest(Number(this.id));
        }
        else {
            await this.issuers.public.revokeRequest(Number(this.id));
        }
    }
    async requestMigrateToPublic(value) {
        if (!this.isPrivate) {
            throw new Error('Certificate is already public.');
        }
        const commitment = { volume: value };
        const { rootHash } = this.prepareEntityCreation(commitment, __1.CommitmentSchema);
        return this.issuers.private.requestMigrateToPublic(Number(this.id), rootHash);
    }
}
exports.Entity = Entity;
exports.createRequestIssue = async (fromTime, toTime, deviceId, configuration, isVolumePrivate = false, forAddress) => {
    const request = new Entity(null, configuration, isVolumePrivate);
    const issuer = isVolumePrivate
        ? configuration.blockchainProperties.issuerLogicInstance.private
        : configuration.blockchainProperties.issuerLogicInstance.public;
    await validateGenerationPeriod(fromTime, toTime, deviceId, configuration, isVolumePrivate);
    const data = await issuer.encodeIssue(fromTime, toTime, deviceId);
    const { logs } = await (forAddress
        ? issuer.requestIssueFor(data, forAddress, utils_general_1.Configuration.getAccount(configuration))
        : issuer.requestIssue(data, utils_general_1.Configuration.getAccount(configuration)));
    request.id = configuration.blockchainProperties.web3.utils
        .hexToNumber(logs[0].topics[2])
        .toString();
    if (configuration.logger) {
        configuration.logger.info(`RequestIssue ${request.id} created`);
    }
    return request.sync();
};
const validateGenerationPeriod = async (fromTime, toTime, deviceId, configuration, isPrivate) => {
    const issuer = isPrivate
        ? configuration.blockchainProperties.issuerLogicInstance.private
        : configuration.blockchainProperties.issuerLogicInstance.public;
    const moment = moment_range_1.extendMoment(Moment);
    const unix = (timestamp) => moment.unix(timestamp);
    const requestIssueIds = await issuer.getRequestIssuesForDevice(deviceId, utils_general_1.Configuration.getAccount(configuration));
    const generationTimeRange = moment.range(unix(fromTime), unix(toTime));
    for (const id of requestIssueIds) {
        const requestIssue = await new Entity(id, configuration, isPrivate).sync();
        if (requestIssue.revoked) {
            continue;
        }
        const requestIssueGenerationRange = moment.range(unix(requestIssue.fromTime), unix(requestIssue.toTime));
        if (generationTimeRange.overlaps(requestIssueGenerationRange)) {
            throw new Error(`Generation period ` +
                `${unix(fromTime).format()} - ${unix(toTime).format()}` +
                ` overlaps with the time period of ` +
                `${unix(requestIssue.fromTime).format()} - ${unix(requestIssue.toTime).format()}` +
                ` of request ${requestIssue.id}.`);
        }
    }
    return true;
};
//# sourceMappingURL=RequestIssue.js.map